-----------------------------------------------------------------------------------------------
-- Client Lua Script for CameraSpeed
-- Copyright RoadBlock
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
 
-----------------------------------------------------------------------------------------------
-- CameraSpeed Module Definition
-----------------------------------------------------------------------------------------------
local CameraSpeed = {} 
local userSetting = {}
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

-----------------------------------------------------------------------------------------------
-- Upvalues
-----------------------------------------------------------------------------------------------
local format = string.format
local tonumber = tonumber 
local tostring = tostring

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CameraSpeed:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function CameraSpeed:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- CameraSpeed OnLoad
-----------------------------------------------------------------------------------------------
function CameraSpeed:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CameraSpeed.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- CameraSpeed OnDocLoaded
-----------------------------------------------------------------------------------------------
function CameraSpeed:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CameraSpeedForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("camspeed", "OnCameraSpeedOn", self)
		self.wndMouseSlider = self.wndMain:FindChild("MouseSpeedSlider")
		self.wndMouseSliderText = self.wndMain:FindChild("MouseSpeedSliderValue")
		self.wndKeyboardSlider = self.wndMain:FindChild("KeyboardSpeedSlider")
		self.wndKeyboardSliderText = self.wndMain:FindChild("KeyboardSpeedSliderValue")

		-- Do additional Addon initialization here
		Print("CameraSpeed Loaded. Use /camspeed for options.")
		
	end
end

-----------------------------------------------------------------------------------------------
-- CameraSpeed Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/cameraspeed"
function CameraSpeed:OnCameraSpeedOn()
	self:SetCurrentValuesOnControls()
	self.wndMain:Invoke() -- show the window
end

function CameraSpeed:SetCurrentValuesOnControls()
	local mouseSpeed = Apollo.GetConsoleVariable("player.turningSpeedMouse")
	self.wndMouseSlider:SetValue(mouseSpeed)
	self.wndMouseSliderText:SetText(format("%0.1f",mouseSpeed))
	local keyboardSpeed = Apollo.GetConsoleVariable("player.turningSpeedKeyboard")
	self.wndKeyboardSlider:SetValue(keyboardSpeed)
	self.wndKeyboardSliderText:SetText(format("%0.1f",keyboardSpeed))
end

-----------------------------------------------------------------------------------------------
-- CameraSpeedForm Functions
-----------------------------------------------------------------------------------------------
-- when the Confirm button is clicked
function CameraSpeed:OnConfirm()
	local mouseSpeed = userSetting.mouseSpeed
	local keyboardSpeed = userSetting.keyboardSpeed
	if mouseSpeed then 
		Apollo.SetConsoleVariable('player.turningSpeedMouse',mouseSpeed)
	end
	if keyboardSpeed then
		Apollo.SetConsoleVariable('player.turningSpeedKeyboard',keyboardSpeed)
	end
	self:SetCurrentValuesOnControls()
	self.wndMain:Close() -- hide the window
end

-- when the Reset button is clicked
function CameraSpeed:OnReset()
	Apollo.ResetConsoleVariable('player.turningSpeedMouse')
	Apollo.ResetConsoleVariable('player.turningSpeedKeyboard')
	self:SetCurrentValuesOnControls()
end


function CameraSpeed:OnKeyboardSpeedChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local keyboardSpeed = tonumber(format("%0.1f",fNewValue))
	userSetting.keyboardSpeed = keyboardSpeed
	self.wndKeyboardSliderText:SetText(keyboardSpeed)
end

function CameraSpeed:OnMouseSpeedChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local mouseSpeed = tonumber(format("%0.1f",fNewValue))
	userSetting.mouseSpeed = mouseSpeed
	self.wndMouseSliderText:SetText(mouseSpeed)
end

function CameraSpeed:OnClose( wndHandler, wndControl, eMouseButton )
	self.wndMain:Close() -- hide the window
end

-----------------------------------------------------------------------------------------------
-- CameraSpeed Instance
-----------------------------------------------------------------------------------------------
local CameraSpeedInst = CameraSpeed:new()
CameraSpeedInst:Init()

